;; -*- lisp -*-
;; See the file LICENCE for licence information.

(in-package :it.bese.ucw)

(def with-macro with-lock-held-on-server (server)
  (multiple-value-prog1
      (progn
        (ucw.rerl.threads.dribble "Entering with-lock-held-on-server for server ~S in thread ~S" server (current-thread))
        (with-recursive-lock-held ((lock-of server))
          -body-))
    (ucw.rerl.threads.dribble "Leaving with-lock-held-on-server for server ~S in thread ~S" server (current-thread))))

(def with-macro with-lock-held-on-application (application)
  (multiple-value-prog1
      (progn
        (ucw.rerl.threads.dribble "Entering with-lock-held-on-application for app ~S in thread ~S" application (current-thread))
        (with-recursive-lock-held ((lock-of application))
          -body-))
    (ucw.rerl.threads.dribble "Leaving with-lock-held-on-application for app ~S in thread ~S" application (current-thread))))

(def with-macro with-lock-held-on-session (session)
  (multiple-value-prog1
      (progn
        (ucw.rerl.threads.dribble "Entering with-lock-held-on-session for ~S in thread ~S" session (current-thread))
        (with-recursive-lock-held ((lock-of session))
          -body-))
    (ucw.rerl.threads.dribble "Leaving with-lock-held-on-session for ~S in thread ~S" session (current-thread))))

;; TODO portable assert that we have the lock on the session
(defun assert-session-lock-held (session)
  #+sbcl
  (assert (eq (sb-thread::mutex-value (lock-of session)) (current-thread)) ()
          "You must have a lock on the session here"))

(defun assert-application-lock-held (application)
  #+sbcl
  (assert (eq (sb-thread::mutex-value (lock-of application)) (current-thread)) ()
          "You must have a lock on the application here"))

(defmacro with-session-variables ((&rest names) &body body)
  "Create local bindings for the listed NAMES and set them to
\(session.value ',name session\). If BODY gracefully completes then
save the values of the local variables back into the session."
  (with-unique-names (session)
    `(let ((,session (context.session *context*)))
      (let (,@(iter (for name in names)
                    (collect `(,name (session.value ',name ,session)))))
        (multiple-value-prog1
            (progn
              ,@body)
          ,@(iter (for name in names)
                  (collect `(setf (session.value ',name ,session) ,name))))))))

(defun send-redirect (target &optional (response *response*))
  (setf (get-header response "Status") +http-moved-temporarily+
        (get-header response "Location") target)
  (<:html
   (<:head
    (<:title "302 - Redirect"))
   (<:body
    (<:p "Page has moved to "
         (<:a :href target (<:ah target))))))

(defun make-standard-ucw-www-root-list ()
  (let ((ucw-home (asdf:component-pathname (asdf:find-system :ucw))))
    (list (cons "static/ucw/" (merge-pathnames "wwwroot/ucw/" ucw-home))
          (cons "static/dojo/" (merge-pathnames "wwwroot/dojo/" ucw-home)))))

(defun make-standard-ucw-tal-dir-list ()
  (let ((ucw-home (asdf:component-pathname (asdf:find-system :ucw))))
    (list (merge-pathnames "wwwroot/ucw/tal/" ucw-home))))

